     module charger_obs_data
 

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! ROUTINES FOR READING OBS data
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! VERSION: 1.0
! WRITTEN BY: NOVELTIS (A.KLONECKI)
!
! CREATION DATE: 29/09/2005
! REVISION DATE:


     contains


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! this subroutine reads the IASI fields necessary for the operational operator:
! lat,lon,date,pressure
      subroutine read_IASI_netcdf(dates,filename        &
                         ,n_lvls                        &
                         ,latmin                        &
                         ,latmax                        &
                         ,lonmin                        &
                         ,lonmax                        &
                         ,presmin                       &
                         ,presmax                       &

                         ,Obs_data                      &
                               )

      use definitions_netcdf_jfl
      use interfaces_netcdf_jfl
!      use convert_dates
      use floats
      use OBS_STRUCTURE


      implicit none
!
! input data
      real(std),dimension(2)     :: dates  !dates of the MOCAGE files in the IASI format
      character*200              :: filename   !name of the IASI data file 
      integer*4                  :: n_lvls     !numb. of obs veritcal levels
      real(std)                  :: latmin,latmax
      real(std)                  :: lonmin,lonmax
      real(std)                  :: presmin,presmax

! output data
      type(OBSERV_DATA)             :: Obs_data
! local variables:
      integer*4         :: n_prf      !number of profiles needed 
      integer           ::NPRF         !total number of profiles in the netcdf file
      integer           ::Index_first  !index in the netcdf file corresponding to dates(1)
      integer           ::Index_last   !index in the netcdf file corresponding to dates(2)

! variables used for reading data from the netcdf file:
      integer*4,dimension(1)                    :: YEAR,MONTH,DAY
      integer                                   :: ncid
! temporary variables used to read the real 4 data in the netcdf file:
      real(kind=low),pointer,dimension(:)   :: lat,lon,pres
      integer*4,pointer,dimension(:)         :: index
! other allocated variables:
      real*8,pointer,dimension(:)   :: IASIday
! indexing variables:
      integer*4                         :: i,i_prf,ip,icount,il

      integer                           :: miss_data
      logical*1                         ::read_only
! start the code

      read_only=.true.

      call open_cdf(ncid,filename,read_only)

      call get_dims(ncid)
      call get_vars(ncid)

      call read_var(ncid,'Year',YEAR(1))
      call read_var(ncid,'Month',MONTH(1))
      call read_var(ncid,'Day',DAY(1))
      call get_dim(ncid,'n_prf',NPRF)

! allocate full dimensions: 
      allocate(IASIday(NPRF))
      call read_var(ncid,'dateIASI',IASIday)
      if (IASIday(1).gt.dates(2).or.   &
          IASIday(NPRF).lt.dates(1)) then
        write(*,*) 'did not find IASI data in the specified file for the specified period'
        write(*,*) 'exiting...'
        call exit(-1)
      end if
! and find out the usefull dimensions corresponding to dates(1) and dates(2)
      i=1
      do while (i.lt.NPRF.and.IASIday(i).lt.dates(1)) 
        i=i+1
      end do
      if (i.eq.NPRF+1) then 
        write(*,*) 'COULDNT FIND THE IASI DATA CORRESPONDING TO THE MODEL TIME 1'
        write(*,*) 'EXITING'
        call exit(-1)
      else
        Index_first=i
      endif
      do while (i.lt.NPRF.and.IASIday(i).lt.dates(2))
        i=i+1
      end do
      Index_last=i
!  found the positions:
      write(*,*) 'FOUND IASI DATA, READING INDICIES ',Index_first,' to ',Index_last

      icount=0
!
      allocate(lat(NPRF))
      allocate(lon(NPRF))
      allocate(index(NPRF))
      allocate(pres(n_lvls))
!
      call read_var(ncid,'Latitude',lat)
      call read_var(ncid,'Longitude',lon)
      call read_var(ncid,'Pressure_Grid',pres)
      do ip=Index_first,Index_last
        if (lat(ip).ge.latmin.and.lat(ip).le.latmax.and.      &
            lon(ip).ge.lonmin.and.lon(ip).le.lonmax) then 
! check if there are pressures in the required pressure range:
            il=1
            do while ((pres(il).lt.presmin.or.pres(il).gt.presmax)  &
                       .and.il.lt.n_lvls)
              il=il+1
            end do
            if (il.le.n_lvls.and.(pres(il).ge.presmin.and.pres(il).le.presmax)) then 
              icount=icount+1
              index(icount)=ip
            end if
        end if
      end do

! n_prf is the number of usefull data: 
      n_prf=icount
      Obs_data%n_prf=n_prf
! allocate the output variables:
      allocate(Obs_data%lat(n_prf))
      allocate(Obs_data%lon(n_prf))
      allocate(Obs_data%date(n_prf))
      allocate(Obs_data%pres(n_lvls,n_prf))

      do ip=1,n_prf
        Obs_data%lat(ip)=lat(index(ip))
        Obs_data%lon(ip)=lon(index(ip))
        Obs_data%date(ip)=IASIday(index(ip))
        do il=1,n_lvls
          if (pres(il).ge.presmin.and. &
              pres(il).le.presmax) then
            Obs_data%pres(il,ip)=pres(il)
          else
            Obs_data%pres(il,ip)=0
          end if
        end do
      end do

      call read_var(ncid,'missing_nodata',miss_data)

      call close_cdf(ncid)

      return
      end subroutine read_IASI_netcdf

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! this subroutine reads the IASI fields necessary for the operational operator:
! lat,lon,date,pressure
      subroutine read_mipas_data(dates,filename         &
                         ,n_lvls                        &
                         ,latmin                        &
                         ,latmax                        &
                         ,lonmin                        &
                         ,lonmax                        &
                         ,presmin                       &
                         ,presmax                       &
                         ,Obs_data            & 
                               )

      use floats
      use OBS_STRUCTURE
      use convert_dates


      implicit none
!
! input data
      real(std),dimension(2)     :: dates      !dates of the MOCAGE files in the IASI format
      character*200              :: filename   !name of the MIPAS data file
      integer*4                  :: n_lvls     !maximum numb. of obs vertical levels
      real(std)                  :: latmin,latmax
      real(std)                  :: lonmin,lonmax
      real(std)                  :: presmin,presmax
! output data
      type(OBSERV_DATA)          :: Obs_data
! local variables:
      integer*4                  :: n_prf      !number of profiles needed
      integer                    ::NPRF         !total number of profiles in the netcdf file
      integer,dimension(5)       :: idateobs
      real*8                     :: sec
      integer                    :: npres
      integer                    :: ninst 
      integer                    :: nbconst 
      CHARACTER*6                :: typemesure
      CHARACTER*5                :: name
      real(kind=low)             :: traceur
      
! indexing variables:           
      integer*4                         :: ip,il,icount,i
      
      integer                           :: iunit
      data iunit/1/

      real(kind=low),pointer,dimension(:)  :: lat
      real(kind=low),pointer,dimension(:)  :: lon
      real(std),pointer,dimension(:)       :: date_obs
      real(kind=low),pointer,dimension(:,:):: pres
      integer*4,pointer,dimension(:)       :: index

      open(iunit,file=filename,form='formatted',err=999)

      read(iunit,*) NPRF
! allocate the temporary variables :
      allocate(lat(NPRF))
      allocate(lon(NPRF))
      allocate(date_obs(NPRF))
      allocate(pres(n_lvls,NPRF))
      allocate(index(NPRF))
! initialize : 
      pres=0.

      read(iunit,*) 
      do ip=1,NPRF
        READ(iunit,102)     &
           (idateobs(i),i=1,5),typemesure,ninst,lat(ip),lon(ip),npres
        sec=idateobs(4)*3600.+idateobs(5)*60
        date_obs(ip)=date_to_dayIASI(idateobs(1),idateobs(2),idateobs(3),sec)
        do il=1,npres
          read(iunit,103) pres(il,ip),nbconst
!          read(iunit,104) name,traceur
          read(iunit,*) 
        end do
      end do
!
      icount=0
      do ip=1,NPRF
        if (date_obs(ip).ge.dates(1).and.            &
            date_obs(ip).le.dates(2).and.            &
            lat(ip).le.latmax.and.                   &
            lat(ip).ge.latmin.and.                   &
            lon(ip).le.lonmax.and.                   &
            lon(ip).ge.lonmin) then 
! check if there are pressures in the required pressure range:
            il=1
            do while ((pres(il,ip).lt.presmin.or.pres(il,ip).gt.presmax)  &
                       .and.il.lt.n_lvls)
              il=il+1
            end do
            if (il.le.n_lvls.and.(pres(il,ip).ge.presmin.and.pres(il,ip).le.presmax)) then 
              icount=icount+1
              index(icount)=ip
            end if
         end if
       end do
       n_prf=icount
       Obs_data%n_prf=n_prf
! allocate the final variables :
       allocate(Obs_data%lat(n_prf))
       allocate(Obs_data%lon(n_prf))
       allocate(Obs_data%date(n_prf))
       allocate(Obs_data%pres(n_lvls,n_prf))
       do ip=1,n_prf
         Obs_data%lat(ip)=lat(index(ip))
         Obs_data%lon(ip)=lon(index(ip))
         Obs_data%date(ip)=date_obs(index(ip))
         do il=1,n_lvls
           if (pres(il,index(ip)).ge.presmin.and. &
               pres(il,index(ip)).le.presmax) then
             Obs_data%pres(il,ip)=pres(il,index(ip))
           else
             Obs_data%pres(il,ip)=0
           end if
         end do
       end do

       return
102    FORMAT(I4,I2.2,I2.2,I2.2,I2.2,1X,A6,1X,I5.5,1X,F6.2,1X,F7.2,1X,I2)
103    FORMAT(F7.0,1X,I2)
104    FORMAT(A5,1X,E12.5)
999    write(*,*) 'error reading the mipas data, exiting'
      call exit(-1)
      end subroutine read_mipas_data

      end module charger_obs_data
