      module convert_dates
! this module contains the 2 routines for converting from date to IASI day and 
! the reverse

contains


!*-------------------------------------------------------------*
!* Comment : convert a date at the format Long Generalised Time
!*           to Julian Days
!*           Julian Day = 0.0 on 01/01/2000 00:00:00.0
!* Input: date at the format Long Generalised Time
!* Output: Julian Days
!* based on the algorithm date2dateJD.f in MNP (JD)
!*-------------------------------------------------------------*
      real*8 function date_to_dayIASI( year,month,day,sec )

! The code start here
      implicit none
!
!     Declaratives
!     ------------
! input variables
      real*8     julian_days
      integer*4  year
      integer*4  month
      integer*4  day
      real*8     sec
!
      logical  LeapYear
      logical IsLeapYear
! local variables

      integer    :: iyear,i
      integer,dimension(12)::DaysByMonth

       DaysByMonth(1)=31
       DaysByMonth(3)=31
       DaysByMonth(4)=30
       DaysByMonth(5)=31
       DaysByMonth(6)=30
       DaysByMonth(7)=31
       DaysByMonth(8)=31
       DaysByMonth(9)=30
       DaysByMonth(10)=31
       DaysByMonth(11)=30
       DaysByMonth(12)=31


!  Initialise julian_days a zero
      julian_days = 0.d0

      do iyear = 2000, year ! loop on the years start on 2000
!        call the leap Year function determination
         LeapYear = IsLeapYear(iyear)
         if ( LeapYear ) then
            DaysByMonth(2) = 29.d0 ! Number of days on February
         else
            DaysByMonth(2) = 28.d0 ! Number of days on February
         end if

         if ( iyear .ne. year ) then
!           the number of days of the year is added
            do i=1,12
              julian_days = julian_days + DaysByMonth(i)
            end do
         end if ! is not this year
      end do ! end of the iyear loop
!     For the last year
      if ( month .gt. 1 ) then
         do month = 1, month - 1
!           add days for past months
            julian_days = julian_days + DaysByMonth(month)
         end do
      end if
!     add days of the current month minus 1st of January
      julian_days = julian_days + day - 1
!     compute the frational part of the Julian Day
      julian_days = julian_days &
          + sec / 86400.d0 
!
      date_to_dayIASI = julian_days

      return
            
      end function date_to_dayIASI
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! this subroutine converts the IASI day (since 01-01-2000 to year, month and day)
   subroutine dayIASI_to_date(dayIASI,year,month,day)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! input variables:
       integer*4            ::dayIASI,year,month,day

! local variables
       logical  LeapYear
       integer            ::julian_days
       integer,dimension(12)::DaysByMonth
       logical IsLeapYear


! The code start here

!  Initialise julian_days a zero

       DaysByMonth(1)=31
       DaysByMonth(3)=31
       DaysByMonth(4)=30
       DaysByMonth(5)=31
       DaysByMonth(6)=30
       DaysByMonth(7)=31
       DaysByMonth(8)=31
       DaysByMonth(9)=30
       DaysByMonth(10)=31
       DaysByMonth(11)=30
       DaysByMonth(12)=31

       julian_days = 0

       year = 2000

       do while (julian_days.lt.dayIASI)
!        call the leap Year function determination
         LeapYear = IsLeapYear(year)
!
         if ( LeapYear ) then
            DaysByMonth(2) = 29.d0 ! Number of days in February
         else
            DaysByMonth(2) = 28.d0 ! Number of days in February
         end if

         month=1
         do while (julian_days.lt.dayIASI.and.month.le.12)
           julian_days = julian_days + DaysByMonth(month)
           month=month+1
         end do

         year=year+1
       end do
             
       month=month-1
       year=year-1
       day=dayIASI-(julian_days-DaysByMonth(month))

       return
       end subroutine dayIASI_to_date

       end module convert_dates

!*-------------------------------------------------------------*
      logical function IsLeapYear( year )

      implicit none
!---------------------------------------------------
!
!     Declaratives
!     ------------
! input variable
      integer*4 year
! local variables
      logical LeapYear
      integer*4 result
! initialise LeapYear variable
      LeapYear = .false.
      result = mod(year, 4) ! a leap year is multiple of 4
      if ( result .eq. 0 ) then
         LeapYear = .true.
      end if
      result = mod(year, 100)    ! a leapyear is multiple of 100
      if ( result .eq. 0 ) then
         LeapYear = .false.
      end if
      result = mod(year, 100)    ! a leapyear is multiple of 400
      if ( result .eq. 0 ) then
         LeapYear = .true.
      end if
            
      IsLeapYear = LeapYear

      return
            
      end function IsLeapYear





