! DEVELOPPED AT NCAR
!
subroutine define_hxy(nlon,nlat,nobs &
                     ,lat_model,lon_model,lat_obs,lon_obs &
                     ,weight,position,discard_obs)
!
  use floats
  implicit none
!
  integer                        , intent(in   ) :: nlon,nlat,nobs
  real(std), dimension(nlat,nlon), intent(in   ) :: lat_model,lon_model
  real(std), dimension(nobs)     , intent(in   ) :: lat_obs,lon_obs
  logical, dimension(nobs)  , intent(inout)      :: discard_obs
  integer, dimension(nobs,4), intent(inout)      :: position
  real(std), dimension(nobs,4), intent(inout)    :: weight
!
! local
!
  integer      :: i,j,k,n,i1,j1,i2,j2,ii,jj,ip
  real(std)    :: dx1,dx2,dy1,dy2
  logical      :: test_lat,test_lon,found_cell,periodic_domain
  logical      :: ll_not_mono
!
! check id domain is periodic in the longitudinal directions
!
!  call check_periodic_domain(nlon,nlat,lon_model,periodic_domain)
!  print *,'periodic_domain = ',periodic_domain
  periodic_domain=.true.
!
! reset output arrays
!
  discard_obs = .true.
  position    = -99
  weight      = 0.
!
! loop over longitude checking monotonicity
!
  LON: DO ip=1,nlon-1
    ll_not_mono = (lon_model(1,ip).gt.lon_model(1,ip+1))
    if (ll_not_mono) exit LON
  END DO LON
!          
! loop over observations
!          
  OBS : do k=1,nobs
!          
! reset flag and cell coordinates
!          
    found_cell = .false.
    i1         = -99
    i2         = -99
    j1         = -99
    j2         = -99
!aak modified to speed up, assumption of a regular grid domain...
    j=1
    LOOP_I : do i=1,nlon-1

        test_lon = .false.
        test_lon = (lon_obs(k).ge.lon_model(j,i  )).and. &
                   (lon_obs(k).lt.lon_model(j,i+1))
        if (test_lon) exit LOOP_I
    end do LOOP_I
    ii=i
    i=1
    LOOP_J : do j=1,nlat-1
        test_lat = .false.
        test_lat = (lat_obs(k).ge.lat_model(j  ,i)).and. &
                   (lat_obs(k).lt.lat_model(j+1,i))
        if (test_lat) exit LOOP_J
    end do LOOP_J
    jj=j
!          
! easy case : found a cell
!          
        if(test_lat.and.test_lon) then
          i1 = ii
          i2 = ii + 1
          j1 = jj
          j2 = jj + 1
          found_cell = .true.
        endif
!
! latitude found, but not longitude => check if periodic
!
        if(test_lat.and.(.not.test_lon)) then
          j1 = j
          j2 = j + 1
          if(periodic_domain) then
	    if(ll_not_mono) then
              if ((lon_obs(k).gt.lon_model(j,ip).and.&
		     lon_obs(k).lt.lon_model(j,ip+1)+360.).or. &
		     (lon_obs(k).gt.lon_model(j,ip)-360..and.&
		     lon_obs(k).lt.lon_model(j,ip+1))) then
	         i1 = ip
		 i2 = ip+1
                 found_cell = .true.
              elseif (lon_obs(k).gt.lon_model(j,nlon).and.&
		     lon_obs(k).lt.lon_model(j,1)) then
		 i1 = nlon
		 i2 =    1
                 found_cell = .true.
              endif
	    else
	      test_lon = (lon_obs(k).lt.lon_model(j,   1)) .or. &
                       (lon_obs(k).ge.lon_model(j,nlon))
              if(test_lon) then
                i1 = nlon
                i2 =    1
                found_cell = .true.
              endif
            endif
          endif
        endif

!
! if no cell was found, that means that the observation
! is outside the model domain, so discard it
!
    if(.not.found_cell) then
      discard_obs(k) = .true.
      cycle OBS
    endif
!
! if you are here, that means that a cell was found.
! One can therefore calculate the weights for each corner
! and make sure that the observation is not discarded.
!
    discard_obs(k) = .false.
    position(k,1) = j1
    position(k,2) = j2
    position(k,3) = i1
    position(k,4) = i2
    IF(ll_not_mono) THEN
	IF(i1.eq.ip) THEN
	    dx1 = ABS(lon_obs(k) - lon_model(j1,i1))
	    dx2 = ABS(lon_obs(k) - lon_model(j1,i2) - 360. )
	ELSE
	    dx1 = ABS(lon_obs(k) - lon_model(j1,i1))
	    dx2 = ABS(lon_obs(k) - lon_model(j1,i2))
	ENDIF
    ELSE
	IF(i1.lt.i2) THEN
	    dx1 = ABS(lon_obs(k) - lon_model(j1,i1))
	    dx2 = ABS(lon_obs(k) - lon_model(j1,i2))
	ELSE
	    dx1 = ABS(lon_obs(k) - lon_model(j1,i1))
	    dx2 = ABS(lon_obs(k) - lon_model(j1,i2) - 360. )
	ENDIF
    ENDIF
    dy1 = ABS(lat_obs(k) - lat_model(j1,i1))
    dy2 = ABS(lat_obs(k) - lat_model(j2,i1))
    IF (dy1.lt.1e-4) THEN
      weight(k,1) = 1.
      weight(k,2) = 0.
    ELSEIF(dy2.lt.1e-4) THEN
      weight(k,1) = 0.
      weight(k,2) = 1.
    ELSE
      weight(k,1) = dy2/(dy1+dy2)
      weight(k,2) = dy1/(dy1+dy2)
    END IF
    IF (dx1.lt.1e-4) THEN
      weight(k,3) = 1.
      weight(k,4) = 0.
    ELSEIF(dx2.lt.1e-4) THEN
      weight(k,3) = 0.
      weight(k,4) = 1.
    ELSE
      weight(k,3) = dx2/(dx1+dx2)
      weight(k,4) = dx1/(dx1+dx2)
    END IF
!
!   if(mod(k,25).eq.0) then
!     print *,k,lon_obs(k),lon_model(j1,i1),lon_model(j1,i2),dx1,dx2
!     print *,k,lat_obs(k),lat_model(j1,i1),lat_model(j2,i1),dy1,dy2
!   endif
!
  end do OBS
!
  return
  end
