      SUBROUTINE ECLOC1(HFUNC,KSEC1,KGRIB,KLENG,KNSPT,KBITS,
     X                     KPLEN,KPR,KRET)
C
C---->
C**** ECLOC1
C
C     Purpose.
C     --------
C
C     GRIB coding/decoding of ECMWF local use of Section 1.
C
C**   Interface.
C     ----------
C
C     CALL ECLOC1 (HFUNC,KSEC1,KGRIB,KLENG,KNSPT,KBITS, KPLEN,KPR,KRET)
C
C
C     Input Parameters.
C     -----------------
C
C     HFUNC      - Requested function.
C                  'C' to code data.
C                  'D' to decode data.
C
C     KSEC1      - Array containing Grib Section 1 data.
C
C     KGRIB      - Array containing Grib coded data.
C
C     KLENG      - Length (words) of KGRIB.
C
C     KNSPT      - Bit number after which insertion or
C                  extraction starts.
C
C     KBITS      - Number of bits in computer word.
C
C     KPLEN      - Pointer to field containing length of Section 1.
C
C     KPR        - Debug print switch.
C                  0   , No printout.
C                  > 0 , Debug printout.
C
C     KRET       - Response to error indicator.
C                  0        , Abort if error encountered.
C                  Non-zero , Return to calling routine
C                             even if error encountered.
C
C     Output Parameters.
C     ------------------
C
C     KSEC1      - Array containing Grib Section 1 data.
C
C     KGRIB      - Array containing Grib coded data.
C
C     KNSPT      - Number of bit after last one inserted or
C                              extracted.
C
C     KRET       - Return code.
C                  0   , No error encountered.
C                  1   , Invalid local use definition.
C                  2   , Error reported by routine INXBIT.
C                  3   , 3d or 4d analysis not implemented.
C
C     Method.
C     -------
C
C     Input data packed/unpacked in accordance with ECMWF usage of
C     local part of section 1 of Grib code.
C
C
C     Externals.
C     ----------
C
C     INXBIT  - insert/extract bit data
C     ABORTX  - abort program
C     CSGNBT  - encode sign bit
C     DSGNBT  - decode sign bit
C     ECDEF1  - encode/decode ECMWF section 1, local usage definition 1
C     ECDEF2  - encode/decode ECMWF section 1, local usage definition 2
C     ECDEF3  - encode/decode ECMWF section 1, local usage definition 3
C     ECDEF4  - encode/decode ECMWF section 1, local usage definition 4
C     ECDEF5  - encode/decode ECMWF section 1, local usage definition 5
C     ECDEF6  - encode/decode ECMWF section 1, local usage definition 6
C     ECDEF7  - encode/decode ECMWF section 1, local usage definition 7
C     ECDEF8  - encode/decode ECMWF section 1, local usage definition 8
C     ECDEF9  - encode/decode ECMWF section 1, local usage definition 9
C     ECDEF10 - encode/decode ECMWF section 1, local usage definition 10
C     ECDEF11 - encode/decode ECMWF section 1, local usage definition 11
C     ECDEF12 - encode/decode ECMWF section 1, local usage definition 12
C     ECDEF13 - encode/decode ECMWF section 1, local usage definition 13
C     ECDEF14 - encode/decode ECMWF section 1, local usage definition 14
C     ECDEF15 - encode/decode ECMWF section 1, local usage definition 15
C     ECDEF16 - encode/decode ECMWF section 1, local usage definition 16
C     ECDEF17 - encode/decode ECMWF section 1, local usage definition 17
C     ECDEF18 - encode/decode ECMWF section 1, local usage definition 18
C     ECDEF19 - encode/decode ECMWF section 1, local usage definition 19
C     ECDEF20 - encode/decode ECMWF section 1, local usage definition 20
C     ECDEF21 - encode/decode ECMWF section 1, local usage definition 21
C     ECDEF22 - encode/decode ECMWF section 1, local usage definition 22
C     ECDEF23 - encode/decode ECMWF section 1, local usage definition 23
C     ECDEF50 - encode/decode ECMWF section 1, local usage definition 50
C     ECDF191 - encode/decode ECMWF section 1, local usage definition 191
#ifdef LITTLE_ENDIAN
C     SWAP4   - swap order of characters in the experiment version
#endif
C
C
C     Reference.
C     ----------
C
C     WMO Manual On Codes for Grib Code.
C
C
C     Comments.
C     ---------
C
C     None.
C
C
C     Author.
C     -------
C
C     J. Hennessy      ECMWF      03:11:92.
C
C
C     Modifications.
C     --------------
C
C     J. Hennessy      ECMWF      21:05:93.
C     Additional parameter KPLEN added. Local usage definitions
C     3 and 4 added.
C
C     J.D.Chambers     ECMWF      18:10:93.
C     Local usage definition 5 added.
C
C     J.D.Chambers     ECMWF      12.05.94
C     ECMWF local usage definition 7 added.
C     (Sensitivity gradient/Trajectory forecast data).
C     Change to use CSGNBT and DSGNBT to code and decode sign bits
C
C     J.D.Chambers     ECMWF      02.06.94
C     Fix handling of local use section 1 for MARS labelling
C
C     J.D.Chambers     ECMWF      11.11.94
C     Fix overwrite of constant zero
C
C     J.D.Chambers     ECMWF      18.01.95
C     Use subroutine calls (ECDEF1, ... ) to handle ECMWF
C     section 1 local usage definitions.
C
C----<
C     -----------------------------------------------------------------|
C*    Section 0 . Definition of variables. Data statements.
C     -----------------------------------------------------------------|
C
      IMPLICIT NONE
C
#include "grbcom.h"
#include "grprs.h"
C
C     Subroutine arguments
C
      CHARACTER*(*) HFUNC
      INTEGER KSEC1, KGRIB, KLENG, KNSPT, KBITS, KPLEN, KPR, KRET
      DIMENSION KGRIB(*)
      DIMENSION KSEC1(*)
C
C     Parameter(s)
C
      INTEGER JPBITS
#ifdef INTEGER_8
      PARAMETER ( JPBITS = 64 )
#else
      PARAMETER ( JPBITS = 32 )
#endif
C
C     Local variables
C
      INTEGER IRET, ILEN, LOOP, ITEMP(1000)
C
      CHARACTER*1   YFUNC
C
      LOGICAL LENCODE, LDEBUG
C
C     Externals
C
      INTEGER E_DEF_1, D_DEF_1
      INTEGER E_DEF_2, D_DEF_2
      INTEGER E_DEF_3, D_DEF_3
      INTEGER E_DEF_4, D_DEF_4
      INTEGER E_DEF_5, D_DEF_5
      INTEGER E_DEF_6, D_DEF_6
      INTEGER E_DEF_7, D_DEF_7
      INTEGER E_DEF_8, D_DEF_8
      INTEGER E_DEF_9, D_DEF_9
      INTEGER E_DEF_10, D_DEF_10
      INTEGER E_DEF_11, D_DEF_11
      INTEGER E_DEF_13, D_DEF_13
      INTEGER E_DEF_14, D_DEF_14
      INTEGER E_DEF_15, D_DEF_15
      INTEGER E_DEF_16, D_DEF_16
      INTEGER E_DEF_17, D_DEF_17
      INTEGER E_DEF_18, D_DEF_18
      INTEGER E_DEF_19, D_DEF_19
      INTEGER E_DEF_20, D_DEF_20
      INTEGER E_DEF_21, D_DEF_21
      INTEGER E_DEF_22, D_DEF_22
      INTEGER E_DEF_23, D_DEF_23
      INTEGER E_DEF_50, D_DEF_50
      INTEGER E_DEF_190, D_DEF_190
      INTEGER E_DEF_191, D_DEF_191
C
C     -----------------------------------------------------------------|
C*    Section 1 . Set initial values.
C     -----------------------------------------------------------------|
C
  100 CONTINUE
C
      LDEBUG = (NDBG.GE.1)
C
      IF( LDEBUG.OR.(KPR.GE.1) ) THEN
        WRITE(GRPRSM,*) 'ECLOC1: Section 1.'
        WRITE(GRPRSM,*) '         Input values used -'
        WRITE(GRPRSM,*) '         HFUNC  = ', HFUNC
        WRITE(GRPRSM,*) '         KNSPT  = ', KNSPT
      ENDIF
C
C     Reset return code to 0, retaining input value to decide
C     on abort / no abort, if error encountered later.
C
      IRET = KRET
      KRET = 0
C
      YFUNC = HFUNC
      LENCODE = ( YFUNC .EQ. 'C' )
C
C     -----------------------------------------------------------------|
C*    Section 2 .  Check input parameters.
C     -----------------------------------------------------------------|
C
  200 CONTINUE
C
      IF( LDEBUG.OR.(KPR.GE.1) ) WRITE(GRPRSM,*) 'ECLOC1: Section 2.'
C
C     Check valid local definition number.
C
      IF( LENCODE.AND.
     X    (KSEC1(37).LT.1.OR.((KSEC1(37).GT.23)
     X    .AND.(KSEC1(37).NE.50)
     X    .AND.(KSEC1(37).NE.190)
     X    .AND.(KSEC1(37).NE.191))))
     X  THEN
        KRET = 1
        WRITE(GRPRSM,*) 'ECLOC1: Invalid local definition: ', KSEC1(37)
        GO TO 900
      ENDIF
C
C*    Octet 41 : Local use definition number.
C     One 8 bit field.
C
      CALL INXBIT (KGRIB,KLENG,KNSPT,KSEC1(37),1,KBITS, 8,YFUNC,KRET)
      IF( KRET.NE.0 ) THEN
        KRET = 2
        WRITE(GRPRSM,9004)
        GO TO 900
      ENDIF
C
      IF( LDEBUG) THEN
        IF( LENCODE ) THEN
          WRITE(GRPRSM,*)
     X      'ECLOC1: Encoding local use definition number ', KSEC1(37)
        ELSE
          WRITE(GRPRSM,*)
     X      'ECLOC1: Decoding local use definition number ', KSEC1(37)
        ENDIF
      ENDIF
C
C     -----------------------------------------------------------------|
C*    Section 3 . Handle individual ECMWF local definitions.
C     -----------------------------------------------------------------|
C
  300 CONTINUE
C
C*    MARS labelling or ensemble forecast data (ECMWF local definition 1)
C
      IF( KSEC1(37).EQ.1 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_1(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          ILEN = E_DEF_1(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 1 section 1 has fixed size (52 bytes)
C
        KNSPT = KNSPT + (52-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C*    Cluster means (ECMWF local definition 2)
C
      IF( KSEC1(37).EQ.2 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_2(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          ILEN = E_DEF_2(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 2 section 1 has fixed size (328 bytes)
C
        KNSPT = KNSPT + (328-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C*    Satellite image data (ECMWF local definition 3)
C
      IF( KSEC1(37).EQ.3 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_3(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          ILEN = E_DEF_3(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 3 section 1 has fixed size (52 bytes)
C
        KNSPT = KNSPT + (52-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C*    If Ocean extensions (ECMWF local definition 4) ...
C     (NB. ECMWF local definition 4 section 1 has variable size)
C
      IF( KSEC1(37).EQ.4 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_4(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ELSE
          ILEN = E_DEF_4(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ENDIF
C
        KRET = 0
        GOTO 900
      ENDIF
C
C     If forecast probabilities (ECMWF local definition 5) ...
C
      IF( KSEC1(37).EQ.5 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_5(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          ILEN = E_DEF_5(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 5 section 1 has fixed size (58 bytes)
C
        KNSPT = KNSPT + (58-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C*    Surface temperature data (ECMWF local definition 6)
C     (NB. ECMWF local definition 6 section 1 has variable size)
C
      IF( KSEC1(37).EQ.6 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_6(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ELSE
          ILEN = E_DEF_6(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ENDIF
C
        KRET = 0
        GOTO 900
      ENDIF
C
C     If sensitivity gradients/trajectory forecasts
C     (ECMWF local definition 7)
C
      IF( KSEC1(37).EQ.7 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_7(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          ILEN = E_DEF_7(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 7 section 1 has fixed size (54 bytes)
C
        KNSPT = KNSPT + (54-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C     If ECMWF re-analysis (definition 8)
C
      IF( KSEC1(37).EQ.8 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_8(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          ILEN = E_DEF_8(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 8 section 1 has fixed size (62 bytes)
C
        KNSPT = KNSPT + (62-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C     If ECMWF singular vector and ensemble perturbation(definition 9)
C
      IF( KSEC1(37).EQ.9 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_9(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          ILEN = E_DEF_9(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 9 section 1 has fixed size (92 bytes)
C
        KNSPT = KNSPT + (92-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C     If ECMWF EPS tubes (definition 10)
C     (NB. ECMWF local definition 10 section 1 has variable size)
C     (The length of local definition 10 section 1 is currently
C       set to 334 bytes)
C
      IF( KSEC1(37).EQ.10 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_10(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ELSE
          ILEN = E_DEF_10(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ENDIF
C
        KRET = 0
        GOTO 900
      ENDIF
C
C     If ECMWF sea surface temperatures (definition 11)
C
      IF( KSEC1(37).EQ.11 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_11(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          ILEN = E_DEF_11(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 11 section 1 has fixed size (72 bytes)
C
        KNSPT = KNSPT + (72-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C     If wave 2D spectra direction and frequency (definition 13)
C     (NB. ECMWF local definition 13 section 1 has variable size)
C
      IF( KSEC1(37).EQ.13 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_13(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ELSE
          ILEN = E_DEF_13(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ENDIF
C
        KRET = 0
        GOTO 900
      ENDIF
C
C     If brightness temperature (definition 14)
C
      IF( KSEC1(37).EQ.14 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_14(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          ILEN = E_DEF_14(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 14 section 1 has fixed size (1080 bytes)
C
        KNSPT = KNSPT + (1080-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C     If ocean ensemble seasonal forecast (definition 15)
C
      IF( KSEC1(37).EQ.15 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_15(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          ILEN = E_DEF_15(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 15 section 1 has fixed size (60 bytes)
C
        KNSPT = KNSPT + (60-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C     Seasonal forecast monthly mean atmosphere data (definition 16)
C
      IF( KSEC1(37).EQ.16 ) THEN
        KRET = IRET
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          KRET = D_DEF_16(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          KRET = E_DEF_16(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 16 section 1 has fixed size (80 bytes)
C
        KNSPT = KNSPT + (80-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C     Sea ice fields used by analysis (definition 17)
C     (NB. ECMWF local definition 17 section 1 has variable size)
C
      IF( KSEC1(37).EQ.17 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_17(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ELSE
          ILEN = E_DEF_17(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ENDIF
C
        KRET = 0
        GOTO 900
      ENDIF
C
C*    Multi-analysis ensemble (ECMWF local definition 18)
C
      IF( KSEC1(37).EQ.18 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          KRET = D_DEF_18(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          KRET = E_DEF_18(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 18 section 1 has fixed size (120 bytes)
C
        KNSPT = KNSPT + (120-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C     Extreme forecast index (definition 19)
C
      IF( KSEC1(37).EQ.19 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          KRET = D_DEF_19(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          KRET = E_DEF_19(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 19 section 1 has fixed size (80 bytes)
C
        KNSPT = KNSPT + (80-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C     4D variational increments (definition 20)
C
      IF( KSEC1(37).EQ.20 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          KRET = D_DEF_20(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          KRET = E_DEF_20(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 20 section 1 has fixed size (52 bytes)
C
        KNSPT = KNSPT + (52-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C     Sensitive area predictions (definition 21)
C
      IF( KSEC1(37).EQ.21 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          KRET = D_DEF_21(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          KRET = E_DEF_21(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 21 section 1 has fixed size (100 bytes)
C
        KNSPT = KNSPT + (100-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C     Coupled atmospheric, wave and ocean models (definition 22)
C
      IF( KSEC1(37).EQ.22 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          KRET = D_DEF_22(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          KRET = E_DEF_22(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 22 section 1 has fixed size (69 bytes)
C
        KNSPT = KNSPT + (69-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C     Coupled atmospheric, wave and ocean means (definition 23)
C     (NB. ECMWF local definition 23section 1 has variable size)
C
      IF( KSEC1(37).EQ.23 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_23(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ELSE
          ILEN = E_DEF_23(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ENDIF
C
        KRET = 0
        GOTO 900
      ENDIF
C
C     Member State data (definition 50)
C
      IF( KSEC1(37).EQ.50 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          KRET = D_DEF_50(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ELSE
          KRET = E_DEF_50(KSEC1(37),KGRIB(1+KNSPT/JPBITS))
        ENDIF
C
C       ECMWF local definition 50 section 1 has fixed size (300 bytes)
C
        KNSPT = KNSPT + (300-40)*8
        KRET = 0
        GOTO 900
      ENDIF
C
C     Multiple ECMWF local definitions (definition 190)
C     (NB. ECMWF local definition 190 section 1 has variable size)
C
      IF( KSEC1(37).EQ.190 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_190(KSEC1(37),KGRIB(1+KNSPT/JPBITS),
     X                     ITEMP,KNSPT)

          DO LOOP = 1, (ILEN-8-KSEC1(44)*2)
            KSEC1(LOOP+44+KSEC1(44)*2) = ITEMP(LOOP)
          ENDDO
C
        ELSE
          ILEN = E_DEF_190(KSEC1(37),KGRIB(1+KNSPT/JPBITS),
     X                     KSEC1(45+KSEC1(44)*2),KNSPT)
        ENDIF
C
        KRET = 0
        GOTO 900
      ENDIF
C
C     Free format data descriptor (definition 191)
C     (NB. ECMWF local definition 191 section 1 has variable size)
C
      IF( KSEC1(37).EQ.191 ) THEN
        KNSPT = KNSPT - 8
        IF( HFUNC.EQ.'D' ) THEN
          ILEN = D_DEF_191(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ELSE
          ILEN = E_DEF_191(KSEC1(37),KGRIB(1+KNSPT/JPBITS),KNSPT)
        ENDIF
C
        KRET = 0
        GOTO 900
      ENDIF
C
C     -----------------------------------------------------------------|
C*    Section 4 . Currently unused ECMWF local definitions.
C     -----------------------------------------------------------------|
C
  400 CONTINUE
C
C     Octet 42 : Class
C     Octet 43 : Type
C     Two 8 bit fields.
C
      CALL INXBIT (KGRIB,KLENG,KNSPT,KSEC1(38),2,KBITS, 8,YFUNC,KRET)
      IF( KRET.NE.0 ) THEN
        KRET = 2
        WRITE(GRPRSM,9004)
        GO TO 900
      ENDIF
C
C*    Octets 44-45 : Stream
C     One 16 bit field.
C
C     Insert/extract fields.
C
      CALL INXBIT (KGRIB,KLENG,KNSPT,KSEC1(40),1,KBITS, 16,YFUNC,KRET)
      IF( KRET.NE.0 ) THEN
        KRET = 2
        WRITE(GRPRSM,9004)
        GO TO 900
      ENDIF
C
C*    Octets 46-49 : Version number or experiment identifier.
C     One 32 bit field.
C
C     Insert/extract field.
C
#ifdef LITTLE_ENDIAN
C
C     Swap character string before encoding
C
Cjdc  IF( LENCODE ) CALL SWAP4(KSEC1(41))
#endif
      CALL INXBIT (KGRIB,KLENG,KNSPT,KSEC1(41),1,KBITS, 32,YFUNC,KRET)
      IF( KRET.NE.0 ) THEN
        KRET = 2
        WRITE(GRPRSM,9004)
        GO TO 900
      ENDIF
C
#ifdef LITTLE_ENDIAN
C
C     Swap character string after encoding (to restore original), or
C     after decoding to put in little-endian order
C
Cjdc  CALL SWAP4(KSEC1(41))
#endif
C
C     If ECMWF Mean, average, etc (definition 12)
C
      IF( KSEC1(37).EQ.12 ) THEN
        KRET = IRET
        CALL ECDEF12(HFUNC, KSEC1, KGRIB, KLENG, KNSPT, KBITS, KPR,KRET)
        GOTO 900
      ENDIF
C
C     Big trouble if this point reached !!
C     Unknown definition encountered.
C
      WRITE(GRPRSM,*) ' Unknown definition for local use = ', KSEC1(37)
      KRET = 3
      GOTO 900
C
C     -----------------------------------------------------------------|
C*    Section 9 . Abort/return to calling routine. Format statements.
C     -----------------------------------------------------------------|
C
  900 CONTINUE
C
      IF( KPR.GE.1 )
     X  WRITE(GRPRSM,*) 'ECLOC1: Output values set, KNSPT  = ', KNSPT
C
C     Abort if requested to do so when an error has been encountered.
C
      IF( (IRET.EQ.0).AND.(KRET.NE.0) ) THEN
        CALL ABORTX ('ECLOC1')
      ELSE
        RETURN
      ENDIF
C
 9004 FORMAT ('ECLOC1: Error reported by routine INXBIT.')
C
      END
