      SUBROUTINE INSCAL (PDATA,KDATA,KLEN,PREF,PSCALE,NBPV)
C
C---->
C**** INSCAL - Vectorise calculation of increments.
C
C     Purpose.
C     --------
C
C     Vectorise calculation of increments.
C
C**   Interface.
C     ----------
C
C     CALL INSCAL (PDATA,KDATA,KLEN,PREF,PSCALE,NBPV)
C
C
C     Input Parameters.
C     -----------------
C
C     PDATA      - Array of floating point values.
C     KLEN       - Number of values to be converted.
C     PREF       - Reference value.
C     PSCALE     - Scale factor.
C     NBPV       - Number of bits per packed value
C
C     Output Parameters.
C     -----------------
C
C     KDATA      - Array of integer increments
C
C     Method.
C     -------
C
C     The reference value is subtracted from each value,
C     and the result is then divided by the scale factor.
C
C     Externals.
C     ----------
C
C     None.
C
C     Reference.
C     ----------
C
C     WMO Manual on Codes re GRIB representation.
C
C     Comments.
C     --------
C
C     PDATA and KDATA are really the same array. This routine
C     is just a device to force vectorisation on the Cray,
C     without the necessity of using another array.
C
C
C     Author.
C     -------
C
C     J. Hennessy     ECMWF     25.06.91
C
C     Modifications.
C     _____________
C
C     J.D.Chambers          ECMWF     21.11.94
C     Use JNINT on VAX
C
C     J.D.Chambers          ECMWF     21.11.94
C     Check number of bits per value is not exceeded
C
C     P.J.Towers            ECMWF     18.07.02
C     Optimised for IBM Power4.
C         -Merged loops
C         -Replaced Nint(x) by Int(x+0.5)
C         -Replaced division by scale by multiplication by reciprocal
C
C----<
C     -----------------------------------------------------------------
C*    Section 0. Definition of variables.
C     -----------------------------------------------------------------
C
#ifdef CRAY
      IMPLICIT NONE
#endif
C
      INTEGER KDATA(KLEN), KLEN, NBPV, LOOP, MAXNBPV
C
      REAL PDATA(KLEN), PREF, PSCALE
      REAL ZRECIP
C
C     -----------------------------------------------------------------
C*    Section 1 . Calculation of increments.
C     -----------------------------------------------------------------
C
  100 CONTINUE
C
      MAXNBPV = 2**NBPV
      ZRECIP = 1.0 / PSCALE
      DO LOOP = 1,KLEN
#ifdef VAX
         KDATA(LOOP) = JNINT ( (PDATA(LOOP) - PREF) / PSCALE )
#else
         KDATA(LOOP) =  INT ( (PDATA(LOOP) - PREF) * ZRECIP + 0.5 )
#endif
         IF( KDATA(LOOP).GE.MAXNBPV ) KDATA(LOOP) = MAXNBPV - 1
         IF( KDATA(LOOP).LT.0 ) KDATA(LOOP) = 0
      ENDDO
C
C     -----------------------------------------------------------------
C*    Section 9. Return to calling routine.
C     -----------------------------------------------------------------
C
  900 CONTINUE
C
      RETURN
      END
