      SUBROUTINE KWPRS1 (KSEC0,KSEC1)
C
C---->
C**** KWPRS1
C
C     Purpose.
C     --------
C
C     Print the information in the Product Definition
C     Section (Section 1) of decoded Washington ensemble GRIB data.
C
C**   Interface.
C     ----------
C
C     CALL KWPRS1 (KSEC0,KSEC1)
C
C
C     Input Parameters.
C     -----------------
C
C     KSEC0 - Array of decoded parameters from Section 0.
C
C     KSEC1 - Array of decoded parameters from Section 1.
C
C
C     Output Parameters.
C     ------------------
C
C     None.
C
C
C     Method.
C     -------
C
C     None.
C
C
C     Externals.
C     ----------
C
C     PRTBIN
C
C
C     Reference.
C     ----------
C
C     WMO Manual on Codes re GRIB Code.
C     See also routine GRIBEX.
C
C
C     Comments.
C     ---------
C
C
C     Washington ensemble product usage of local part of section 1 of
C     GRIB:
C
C     KSEC1(37) decoded from Octet 41.
C     Identifies the application:
C                1   Ensemble.
C
C     KSEC1(38) decoded from Octet 42.
C     Type:
C                1   Unperturbed control forecast,
C                2   Individual negatively perturbed forecast,
C                3   Individual positively perturbed forecast,
C                4   Cluster,
C                5   Whole ensemble.
C
C     KSEC1(39) decoded from Octet 43.
C     Identification number:
C                1   If byte 42=1, identifies high resolution.
C                2   If byte 42=1, identifies low resolution.
C                3   Etc.(!?).
C
C     KSEC1(40) decoded from Octet 44.
C     Products:
C                1   Full field (individual forecast)/unweighted mean.
C                2   Weighted mean.
C                11  Standard deviation wrt ensemble mean.
C                11  Standard deviation wrt ensemble mean, normalized.
C
C     KSEC1(41) decoded from Octet 45.
C     Spatial smoothing of product (number of highest total
C     wavenumber included):
C                255 Original resolution retained.
C
C     KSEC1(42) decoded from Octet 46 : Dummy.
C
C
C     Author.
C     -------
C
C     J.D.Chambers   ECMWF 09.10.95
C
C
C     Modifications.
C     --------------
C
C     None.
C
C
C----<
C     -----------------------------------------------------------------
C*    Section 0. Definition of variables.
C     -----------------------------------------------------------------
C
      IMPLICIT NONE
C
#include "grprs.h"
C
C     Parameters
      INTEGER KSEC0
      INTEGER KSEC1
      DIMENSION KSEC0(*)
      DIMENSION KSEC1(*)
C
C     Local variables
C
C     -----------------------------------------------------------------
C*    Section 1 . Print required information.
C     ----------------------------------------------------------------- C
  100 CONTINUE
C
C     -----------------------------------------------------------------
C*    Section 2 . Print Washington ensemble product information.
C     -----------------------------------------------------------------
C
  200 CONTINUE
C
C     Application Identifier.
      WRITE(GRPRSM,9000) KSEC1(37)
C
C     Type.
      IF(KSEC1(38).GT.5) WRITE(GRPRSM,9100) KSEC1(38)
      IF(KSEC1(38).EQ.1) WRITE(GRPRSM,9101) KSEC1(38)
      IF(KSEC1(38).EQ.2) WRITE(GRPRSM,9102) KSEC1(38)
      IF(KSEC1(38).EQ.3) WRITE(GRPRSM,9103) KSEC1(38)
      IF(KSEC1(38).EQ.4) WRITE(GRPRSM,9104) KSEC1(38)
      IF(KSEC1(38).EQ.5) WRITE(GRPRSM,9105) KSEC1(38)
C
C     Identification number.
      IF(KSEC1(38).NE.1) THEN
        WRITE(GRPRSM,9200) KSEC1(39)
      ELSE
        IF(KSEC1(39).EQ.1) THEN
          WRITE(GRPRSM,9201) KSEC1(39)
        ELSE IF(KSEC1(39).EQ.2) THEN
          WRITE(GRPRSM,9202) KSEC1(39)
        ELSE
          WRITE(GRPRSM,9200) KSEC1(39)
        ENDIF
      ENDIF
C
C     Product.
      IF(KSEC1(40).EQ.1) THEN
        WRITE(GRPRSM,9301) KSEC1(40)
      ELSE IF(KSEC1(40).EQ.2) THEN
        WRITE(GRPRSM,9302) KSEC1(40)
      ELSE IF(KSEC1(40).EQ.11) THEN
        WRITE(GRPRSM,9311) KSEC1(40)
      ELSE IF(KSEC1(40).EQ.12) THEN
        WRITE(GRPRSM,9312) KSEC1(40)
      ELSE
        WRITE(GRPRSM,9300) KSEC1(40)
      ENDIF
C
C     Spatial smoothing of product.
      IF( KSEC1(41).EQ.255) THEN
        WRITE(GRPRSM,9400) KSEC1(41)
      ELSE
        WRITE(GRPRSM,9401) KSEC1(41)
      ENDIF
C
C     -----------------------------------------------------------------
C*    Section 9 . Format statements. Return to calling routine.
C     -----------------------------------------------------------------
C
  900 CONTINUE
C
      RETURN
C
 9000 FORMAT(1H ,'Application Identifier.(1=Ensemble)  ',I9)
C
 9100 FORMAT(1H ,'Type:                                ',I9)
 9101 FORMAT(1H ,'Type: Unperturbed control forecast.  ',I9)
 9102 FORMAT(1H ,'Type: Individual -vely perturbed fc. ',I9)
 9103 FORMAT(1H ,'Type: Individual +vely perturbed fc. ',I9)
 9104 FORMAT(1H ,'Type: Cluster.                       ',I9)
 9105 FORMAT(1H ,'Type: Whole ensemble.                ',I9)
C
 9200 FORMAT(1H ,'Identification number.               ',I9)
 9201 FORMAT(1H ,'Identification number: (1=high res)  ',I9)
 9202 FORMAT(1H ,'Identification number: (2=low res)   ',I9)
C
 9300 FORMAT(1H ,'Product.                             ',I9)
 9301 FORMAT(1H ,'Full field/unweighted mean.          ',I9)
 9302 FORMAT(1H ,'Weighted mean.                       ',I9)
 9311 FORMAT(1H ,'Standard deviation wrt ensemble mean.',I9)
 9312 FORMAT(1H ,'Stand.dev. wrt ens. mean, normalized.',I9)
C
 9400 FORMAT(1H ,'Original spatial smoothing retained. ',I9)
 9401 FORMAT(1H ,'Spatial smoothing of product:        ',I9)
C
      END
