      SUBROUTINE MORTHO( V, AV, BV, HES1, MITER, NEQ, ITS )

      USE control_vectors
C
C**** *MORTHO*   - Routine to orthogolize a vector 
C                  
C
C     Purpose.
C     --------
C        Orthonormalize a vector v(.,its) with respect to the B 
C        inner product and then determine Av(1,its) and Bv(.,its). 
C        The B operator is the symmetric positive operator in Ax=cBx
C**   Interface.
C     ----------
C        *CALL* *MORTHO( V, AV, BV, HES1, MITER, NEQ, ITS )
C
C        Explicit arguments :
C        --------------------
C        input  : v,av,bv,hes1,neq,its
C        output : hes1,av,bv
C
C        Implicit arguments :
C        --------------------
C        None
C
C     Method.
C     -------
C        See documentation
C
C     Externals.
C     ----------
C        None
C
C     Reference.
C     ----------
C        See documentation
C
C     Author.
C     -------
C        Martin van Gijzen *Utrecht University*
C
C     Modifications.
C     --------------
C        Original : 95-10-5
C
c**********************************************************************
c
      INTEGER          NEQ, ITS, MITER
      REAL             V(NEQ,MITER), AV(NEQ,MITER), BV(NEQ,MITER), 
     $                 HES1(MITER,MITER),XS(NEQ)
      TYPE (control_vector) :: ZVAZY,ZVAZH
C
C...     BLAS:
      REAL             DDOT
      REAL             T
      INTEGER          J
c
c...     Orthogonalize the new basis vector:
      DO J = 1, ITS-1
         T = DDOT( NEQ, BV(1,J), 1, V(1,ITS), 1 )
         CALL DAXPY( NEQ, -T, V(1,J), 1, V(1,ITS), 1 )
      ENDDO
c    
c...     Normalize in B-norm:

      CALL ALLOCATE_CTLVEC(ZVAZY)
      CALL ALLOCATE_CTLVEC(ZVAZH)

      ZVAZY = V(1:NEQ,ITS)
      CALL OPM(ZVAZY,ZVAZH)
      CALL ALLGATHER_CTLVEC_IFSAUX(ZVAZH, NEQ, BV(1:NEQ,ITS))

      CALL DEALLOCATE_CTLVEC(ZVAZY)
      CALL DEALLOCATE_CTLVEC(ZVAZH)
 
      T = DDOT( NEQ, BV(1,ITS), 1, V(1,ITS), 1 )
      IF ( T .LE. 0. ) THEN 
         WRITE(0,*) ' DEPENDENT BASIS-VECTORS '
         STOP 
      ELSE IF ( T .LE. 1.D-12 ) THEN
         WRITE(0,*) ' NEARLY DEPENDENT BASIS-VECTORS '
         T = 1./SQRT( T )
         CALL DSCAL( NEQ, T, V(1,ITS), 1 )
         DO J = 1, ITS-1
            T = DDOT( NEQ, BV(1,J), 1, V(1,ITS), 1 )
            CALL DAXPY( NEQ, -T, V(1,J), 1, V(1,ITS), 1 )
         ENDDO

         CALL ALLOCATE_CTLVEC(ZVAZY)
         CALL ALLOCATE_CTLVEC(ZVAZH)

         ZVAZY = V(1:NEQ,ITS)
         CALL OPM(ZVAZY,ZVAZH)
         CALL ALLGATHER_CTLVEC_IFSAUX(ZVAZH, NEQ, BV(1:NEQ,ITS))

         CALL DEALLOCATE_CTLVEC(ZVAZY)
         CALL DEALLOCATE_CTLVEC(ZVAZH)

         T = DDOT( NEQ, BV(1,ITS), 1, V(1,ITS), 1 )
      END IF
      IF ( T .LE. 0. ) THEN
         WRITE(0,*) ' DEPENDENT BASIS-VECTORS '
         STOP
      END IF
      T = 1./SQRT( T )
      CALL DSCAL( NEQ, T, V(1,ITS), 1 )
      CALL DSCAL( NEQ, T,BV(1,ITS), 1 )
c
c...     Compute Av
      CALL ALLOCATE_CTLVEC(ZVAZY)
      CALL ALLOCATE_CTLVEC(ZVAZH)

      ZVAZY = V(1:NEQ,ITS)
      CALL OPK(ZVAZY,ZVAZH)
      CALL ALLGATHER_CTLVEC_IFSAUX(ZVAZH, NEQ, AV(1:NEQ,ITS))

      CALL DEALLOCATE_CTLVEC(ZVAZY)
      CALL DEALLOCATE_CTLVEC(ZVAZH)
c
c...     Compute next row and column of projected matrix
      DO J = 1, ITS
         HES1(J,ITS) = DDOT( NEQ, V(1,J), 1, AV(1,ITS), 1 )
      ENDDO
c
      END
